@@ -11,9 +11,10 @@
 #include <gconf/gconf.h>
 #include <gconf/gconf-client.h>
 #include <jg_jnu.h>
+#include <gtk_java.h>
+
+#include "org_gnu_gconf_ConfClient.h"
 
-#ifndef _Included_org_gnu_gconf_ConfClient
-#define _Included_org_gnu_gconf_ConfClient
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -24,7 +25,7 @@ extern "C" {
 JNIEXPORT jobject JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1default
   (JNIEnv *env, jclass cls)
 {
-	return getHandleFromPointer(env, gconf_client_get_default());
+	return getGObjectHandle(env, G_OBJECT(gconf_client_get_default()));
 }
 
 /*
@@ -35,41 +36,37 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1for_
   (JNIEnv *env, jclass cls, jobject engine)
 {
 	GConfEngine* engine_g = (GConfEngine*)getPointerFromHandle(env, engine);
-	return getHandleFromPointer(env, gconf_client_get_for_engine(engine_g));
+	return getGObjectHandle(env, G_OBJECT(gconf_client_get_for_engine(engine_g)));
 }
 
 /*
  * Class:     org_gnu_gconf_ConfClient
  * Method:    gconf_client_add_dir
  */
-JNIEXPORT void JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1add_1dir
-  (JNIEnv *env, jclass cls, jobject client, jstring dir, jint preload, jintArray exception)
+JNIEXPORT jobject JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1add_1dir
+  (JNIEnv *env, jclass cls, jobject client, jstring dir, jint preload)
 {
 	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, dir, NULL);
-	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
+	GError* err = NULL;
 	gconf_client_add_dir(client_g, str, (GConfClientPreloadType)preload, &err);
 	(*env)->ReleaseStringUTFChars(env, dir, str);
-	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
-	if (err != NULL)
-		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
+	return getStructHandle(env, err, NULL, (JGFreeFunc) g_error_free);
 }
 
 /*
  * Class:     org_gnu_gconf_ConfClient
  * Method:    gconf_client_remove_dir
  */
-JNIEXPORT void JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1remove_1dir
-  (JNIEnv *env, jclass cls, jobject client, jstring dir, jintArray exception)
+JNIEXPORT jobject JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1remove_1dir
+  (JNIEnv *env, jclass cls, jobject client, jstring dir)
 {
 	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, dir, NULL);
-	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
+	GError* err = NULL;
 	gconf_client_remove_dir(client_g, str, &err);
 	(*env)->ReleaseStringUTFChars(env, dir, str);
-	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
-	if (err != NULL) 
-		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
+	return getStructHandle(env, err, NULL, (JGFreeFunc) g_error_free);
 }
 
 typedef struct 
@@ -77,17 +74,22 @@ typedef struct
 	JNIEnv* env;
 	jobject obj;
 	jmethodID methodID;
-	jstring namespace;
+	jstring nameSpace;
 } NotifyData;
 
-void notifyCallback(GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer data)
+static void notifyCallback(GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer data)
 {
 	NotifyData *nd = (NotifyData*)data;
-	(*nd->env)->CallVoidMethod(nd->env, nd->obj, nd->methodID, (jint)entry, nd->namespace);
+	jobject entry_g = getStructHandle(nd->env, entry,
+			(JGCopyFunc) gconf_entry_copy, (JGFreeFunc) gconf_entry_unref);
+	(*nd->env)->CallVoidMethod(nd->env, nd->obj, nd->methodID, entry_g, nd->nameSpace);
 }
 
-void notifyFree(gpointer data)
+static void notifyFree(gpointer data)
 {
+	NotifyData *nd = (NotifyData*)data;
+	(*nd->env)->DeleteGlobalRef(nd->env, nd->obj);
+	(*nd->env)->DeleteGlobalRef(nd->env, nd->nameSpace);
 	g_free((NotifyData*)data);
 }
 
@@ -96,30 +98,31 @@ void notifyFree(gpointer data)
  * Method:    gconf_client_notify_add
  */
 JNIEXPORT jint JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1notify_1add
-  (JNIEnv *env, jobject obj, jobject client, jstring namespace, jintArray exception)
+  (JNIEnv *env, jobject obj, jobject client, jstring nameSpace, jobject exception)
 {
 	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	static jmethodID methodID = NULL;
 	jclass cls = (*env)->GetObjectClass(env, obj);
-	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
-	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, namespace, NULL);
+	GError* err = NULL;
+	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, nameSpace, NULL);
 	NotifyData *data;
 	guint ret;
 	if (methodID == NULL) {
-		methodID = (*env)->GetMethodID(env, cls, "notifyCallback", "(Lorg/gnu/javagnome/Handle;Ljava/lang/String;)V");
+		methodID = (*env)->GetMethodID(env, cls, "notifyCallback",
+				"(Lorg/gnu/glib/Handle;Ljava/lang/String;)V");
 		if (methodID == NULL)
 			return -1;
 	}
 	data = g_new(NotifyData, 1);
 	data->env = env;
-	data->obj = obj;
+	data->obj = (*env)->NewGlobalRef(env, obj);
 	data->methodID = methodID;
-	data->namespace = namespace;
+	data->nameSpace = (*env)->NewGlobalRef(env, nameSpace);
 	ret = gconf_client_notify_add(client_g, str, notifyCallback, data, notifyFree, &err);
-	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
-	(*env)->ReleaseStringUTFChars(env, namespace, str);
-	if (err != NULL)
-		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
+	(*env)->ReleaseStringUTFChars(env, nameSpace, str);
+	if (err != NULL)	{
+		updateStructHandle(env, exception, err, (JGFreeFunc) g_error_free);
+	}
 	return ret;
 }
 
@@ -151,14 +154,13 @@ JNIEXPORT void JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1clear_1cache
  * Signature: (ILjava/lang/String;I[I)V
  */
 JNIEXPORT void JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1preload
-  (JNIEnv *env, jclass cls, jobject client, jstring dir, jint type, jintArray exception)
+  (JNIEnv *env, jclass cls, jobject client, jstring dir, jint type, jobject exception)
 {
 	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, dir, NULL);
-	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
+	GError* err = NULL;
 	gconf_client_preload(client_g, str, (GConfClientPreloadType)type, &err);
 	(*env)->ReleaseStringUTFChars(env, dir, str);
-	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
 	if (err != NULL)
 		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
 }
@@ -168,15 +170,14 @@ JNIEXPORT void JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1preload
  * Method:    gconf_client_set
  */
 JNIEXPORT void JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1set
-  (JNIEnv *env, jclass cls, jobject client, jstring key, jobject val, jintArray exception)
+  (JNIEnv *env, jclass cls, jobject client, jstring key, jobject val, jobject exception)
 {
 	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
 	GConfValue* val_g = (GConfValue*)getPointerFromHandle(env, val);
-	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
+	GError* err = NULL;
 	gconf_client_set(client_g, str, val_g, &err);
 	(*env)->ReleaseStringUTFChars(env, key, str);
-	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
 	if (err != NULL)
 		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
 }
@@ -186,17 +187,17 @@ JNIEXPORT void JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1set
  * Method:    gconf_client_get
  */
 JNIEXPORT jobject JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get
-  (JNIEnv *env, jclass cls, jobject client, jstring key, jintArray exception)
+  (JNIEnv *env, jclass cls, jobject client, jstring key, jobject exception)
 {
 	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
-	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
+	GError* err = NULL;
 	GConfValue* val = gconf_client_get(client_g, str, &err);
 	(*env)->ReleaseStringUTFChars(env, key, str);
-	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
-	if (err != NULL)
-		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
-	return getHandleFromPointer(env, val);
+	if (err != NULL)	{
+		updateStructHandle(env, exception, err, (JGFreeFunc) g_error_free);
+	}
+	return getStructHandle(env, val, NULL, (JGFreeFunc) gconf_value_free);
 }
 
 /*
@@ -204,17 +205,17 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get
  * Method:    gconf_client_get_without_default
  */
 JNIEXPORT jobject JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1without_1default
-  (JNIEnv *env, jclass cls, jobject client, jstring key, jintArray exception)
+  (JNIEnv *env, jclass cls, jobject client, jstring key, jobject exception)
 {
 	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
-	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
+	GError* err = NULL;
 	GConfValue* val = gconf_client_get_without_default(client_g, str, &err);
 	(*env)->ReleaseStringUTFChars(env, key, str);
-	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
-	if (err != NULL)
-		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
-	return getHandleFromPointer(env, val);
+	if (err != NULL)	{
+		updateStructHandle(env, exception, err, (JGFreeFunc) g_error_free);
+	}
+	return getStructHandle(env, val, NULL, (JGFreeFunc) gconf_value_free);
 }
 
 /*
@@ -222,19 +223,20 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1with
  * Method:    gconf_client_get_entry
  */
 JNIEXPORT jobject JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1entry
-  (JNIEnv *env, jclass cls, jobject client, jstring key, jstring locale, jboolean schemaDefault, jintArray exception)
+  (JNIEnv *env, jclass cls, jobject client, jstring key, jstring locale,
+  		jboolean schemaDefault, jobject exception)
 {
 	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str1 = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
 	gchar *str2 = (gchar*)(*env)->GetStringUTFChars(env, locale, NULL);
-	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
+	GError* err = NULL;
 	GConfEntry* val = gconf_client_get_entry(client_g, str1, str2, (gboolean)schemaDefault, &err);
 	(*env)->ReleaseStringUTFChars(env, key, str1);
 	(*env)->ReleaseStringUTFChars(env, locale, str2);
-	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
-	if (err != NULL)
-		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
-	return getHandleFromPointer(env, val);
+	if (err != NULL)	{
+		updateStructHandle(env, exception, err, (JGFreeFunc) g_error_free);
+	}
+	return getStructHandle(env, val, NULL, (JGFreeFunc) gconf_entry_unref);
 }
 
 /*
@@ -242,17 +244,17 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1entr
  * Method:    gconf_client_get_default_from_schema
  */
 JNIEXPORT jobject JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1default_1from_1schema
-  (JNIEnv *env, jclass cls, jobject client, jstring key, jintArray exception)
+  (JNIEnv *env, jclass cls, jobject client, jstring key, jobject exception)
 {
 	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
-	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
+	GError* err = NULL;
 	GConfValue* val = gconf_client_get_default_from_schema(client_g, str, &err);
 	(*env)->ReleaseStringUTFChars(env, key, str);
-	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
-	if (err != NULL)
-		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
-	return getHandleFromPointer(env, val);
+	if (err != NULL)	{
+		updateStructHandle(env, exception, err, (JGFreeFunc) g_error_free);
+	}
+	return getStructHandle(env, val, NULL, (JGFreeFunc) gconf_value_free);
 }
 
 /*
@@ -260,37 +262,42 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1defa
  * Method:    gconf_client_unset
  */
 JNIEXPORT jboolean JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1unset
-  (JNIEnv *env, jclass cls, jobject client, jstring key, jintArray exception)
+  (JNIEnv *env, jclass cls, jobject client, jstring key, jobject exception)
 {
 	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
-	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
+	GError* err = NULL;
 	gboolean val = gconf_client_unset(client_g, str, &err);
 	(*env)->ReleaseStringUTFChars(env, key, str);
-	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
-	if (err != NULL)
-		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
+	if (err != NULL)	{
+		updateStructHandle(env, exception, err, (JGFreeFunc) g_error_free);
+	}
 	return (jboolean)val;
 }
 
+static jobject getGConfEntry(JNIEnv *env, gpointer entry)
+{
+	return getStructHandle(env, entry, NULL, (JGFreeFunc) gconf_entry_unref);
+}
+
 /*
  * Class:     org_gnu_gconf_ConfClient
  * Method:    gconf_client_all_entries
  */
 JNIEXPORT jobjectArray JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1all_1entries
-  (JNIEnv *env, jclass cls, jobject client, jstring dir, jintArray exception)
+  (JNIEnv *env, jclass cls, jobject client, jstring dir, jobject exception)
 {
 	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, dir, NULL);
-	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
+	GError* err = NULL;
 	GSList* val = gconf_client_all_entries(client_g, str, &err);
-	if (NULL == val)
-		return NULL;
 	(*env)->ReleaseStringUTFChars(env, dir, str);
-	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
-	if (err != NULL)
-		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
-	return getHandleArrayFromGSList(env, val);
+	if (err != NULL)	{
+		updateStructHandle(env, exception, err, (JGFreeFunc) g_error_free);
+	}
+	if (val == NULL)
+		return NULL;
+	return getStructHandlesFromGSList(env, val, (GetHandleFunc) getGConfEntry);
 }
 
 /*
@@ -298,32 +305,38 @@ JNIEXPORT jobjectArray JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1all_
  * Method:    gconf_client_all_dirs
  */
 JNIEXPORT jobjectArray JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1all_1dirs
-  (JNIEnv *env, jclass cls, jobject client, jstring dir, jintArray exception)
+  (JNIEnv *env, jclass cls, jobject client, jstring dir, jobject exception)
 {
-	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
-	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, dir, NULL);
-	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
-	GSList* val = gconf_client_all_dirs(client_g, str, &err);
 	int i = 0;
 	guint len;
 	jclass strCls;
 	jobjectArray ar;
-	if (NULL == val)
+	GError* err;
+	GSList* val;
+	GSList* item;
+	GConfClient* client_g;
+	gchar *str;
+	
+	client_g = (GConfClient*)getPointerFromHandle(env, client);
+	str = (gchar*)(*env)->GetStringUTFChars(env, dir, NULL);
+	err = NULL;
+	val = gconf_client_all_dirs(client_g, str, &err);
+	(*env)->ReleaseStringUTFChars(env, dir, str);
+	if (val == NULL)
 		return NULL;
 	len = g_slist_length(val);
 	strCls = (*env)->FindClass(env, "java/lang/String");
-	if (NULL == strCls)
+	if (strCls == NULL)
 		return NULL;
 	ar = (*env)->NewObjectArray(env, len, strCls, NULL);
-	for (i = 0; i < len; i++) {
-		GSList *item = g_slist_nth(val, i);
+	item = val;
+	for (i = 0; item != NULL; item = item->next, ++i) {
 	 	jstring str = (*env)->NewStringUTF(env, (gchar*)item->data);
 		(*env)->SetObjectArrayElement(env, ar, i, str);
 	}
-	(*env)->ReleaseStringUTFChars(env, dir, str);
-	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
-	if (err != NULL)
-		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
+	if (err != NULL)	{
+		updateStructHandle(env, exception, err, (JGFreeFunc) g_error_free);
+	}
 	return ar;
 }
 
@@ -331,13 +344,13 @@ JNIEXPORT jobjectArray JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1all_
  * Class:     org_gnu_gconf_ConfClient
  * Method:    gconf_client_suggest_sync
  */
-JNIEXPORT void JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1suggest_1sync
-  (JNIEnv *env, jclass cls, jobject client, jintArray exception)
+JNIEXPORT jobject JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1suggest_1sync
+  (JNIEnv *env, jclass cls, jobject client)
 {
 	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
-	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
+	GError* err = NULL;
 	gconf_client_suggest_sync(client_g, &err);
-	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
+	return getStructHandle(env, err, NULL, (JGFreeFunc) g_error_free);
 }
 
 /*
@@ -345,16 +358,16 @@ JNIEXPORT void JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1suggest_1syn
  * Method:    gconf_client_dir_exists
  */
 JNIEXPORT jboolean JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1dir_1exists
-  (JNIEnv *env, jclass cls, jobject client, jstring dir, jintArray exception)
+  (JNIEnv *env, jclass cls, jobject client, jstring dir, jobject exception)
 {
 	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, dir, NULL);
-	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
+	GError* err = NULL;
 	gboolean val = gconf_client_dir_exists(client_g, str, &err);
 	(*env)->ReleaseStringUTFChars(env, dir, str);
-	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
-	if (err != NULL)
-		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
+	if (err != NULL)	{
+		updateStructHandle(env, exception, err, (JGFreeFunc) g_error_free);
+	}
 	return (jboolean)val;
 }
 
@@ -363,16 +376,16 @@ JNIEXPORT jboolean JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1dir_1exi
  * Method:    gconf_client_get_float
  */
 JNIEXPORT jdouble JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1float
-  (JNIEnv *env, jclass cls, jobject client, jstring key, jintArray exception)
+  (JNIEnv *env, jclass cls, jobject client, jstring key, jobject exception)
 {
 	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
-	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
+	GError* err = NULL;
 	gdouble val = gconf_client_get_float(client_g, str, &err);
 	(*env)->ReleaseStringUTFChars(env, key, str);
-	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
-	if (err != NULL)
-		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
+	if (err != NULL)	{
+		updateStructHandle(env, exception, err, (JGFreeFunc) g_error_free);
+	}
 	return (jdouble)val;
 }
 
@@ -381,16 +394,16 @@ JNIEXPORT jdouble JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1floa
  * Method:    gconf_client_get_int
  */
 JNIEXPORT jint JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1int
-  (JNIEnv *env, jclass cls, jobject client, jstring key, jintArray exception)
+  (JNIEnv *env, jclass cls, jobject client, jstring key, jobject exception)
 {
 	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
-	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
+	GError* err = NULL;
 	gint val = gconf_client_get_int(client_g, str, &err);
 	(*env)->ReleaseStringUTFChars(env, key, str);
-	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
-	if (err != NULL)
-		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
+	if (err != NULL)	{
+		updateStructHandle(env, exception, err, (JGFreeFunc) g_error_free);
+	}
 	return (jint)val;
 }
 
@@ -399,15 +412,16 @@ JNIEXPORT jint JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1int
  * Method:    gconf_client_get_string
  */
 JNIEXPORT jstring JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1string
-  (JNIEnv *env, jclass cls, jobject client, jstring key, jintArray exception)
+  (JNIEnv *env, jclass cls, jobject client, jstring key, jobject exception)
 {
 	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
-	GError *err = NULL;
+	GError* err = NULL;
 	gchar* val = gconf_client_get_string(client_g, str, &err);
 	(*env)->ReleaseStringUTFChars(env, key, str);
-	if (err != NULL)
-		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
+	if (err != NULL)	{
+		updateStructHandle(env, exception, err, (JGFreeFunc) g_error_free);
+	}
 	return (*env)->NewStringUTF(env, val);
 }
 
@@ -416,16 +430,16 @@ JNIEXPORT jstring JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1stri
  * Method:    gconf_client_get_bool
  */
 JNIEXPORT jboolean JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1bool
-  (JNIEnv *env, jclass cls, jobject client, jstring key, jintArray exception)
+  (JNIEnv *env, jclass cls, jobject client, jstring key, jobject exception)
 {
 	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
-	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
+	GError* err = NULL;
 	gboolean val = gconf_client_get_bool(client_g, str, &err);
 	(*env)->ReleaseStringUTFChars(env, key, str);
-	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
-	if (err != NULL)
-		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
+	if (err != NULL)	{
+		updateStructHandle(env, exception, err, (JGFreeFunc) g_error_free);
+	}
 	return (jboolean)val;
 }
 
@@ -434,17 +448,17 @@ JNIEXPORT jboolean JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1boo
  * Method:    gconf_client_get_schema
  */
 JNIEXPORT jobject JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1schema
-  (JNIEnv *env, jclass cls, jobject client, jstring key, jintArray exception)
+  (JNIEnv *env, jclass cls, jobject client, jstring key, jobject exception)
 {
 	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
-	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
+	GError* err = NULL;
 	GConfSchema* val = gconf_client_get_schema(client_g, str, &err);
 	(*env)->ReleaseStringUTFChars(env, key, str);
-	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
-	if (err != NULL) 
-		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
-	return getHandleFromPointer(env, val);
+	if (err != NULL)	{
+		updateStructHandle(env, exception, err, (JGFreeFunc) g_error_free);
+	}
+	return getStructHandle(env, val, NULL, (JGFreeFunc) gconf_schema_free);
 }
 
 /*
@@ -452,20 +466,31 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1sche
  * Method:    gconf_client_get_list
  */
 JNIEXPORT jobjectArray JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1list
-  (JNIEnv *env, jclass cls, jobject client, jstring key, jint type, jintArray exception)
+  (JNIEnv *env, jclass cls, jobject client, jstring key, jint type, jobject exception)
 {
-	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
-	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
-	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
-	GConfValueType vt = (GConfValueType)type;
-	GSList* val = gconf_client_get_list(client_g, str, vt, &err);
+	GConfClient* client_g;
+	GError* err;
+	gchar *str;
+	GConfValueType vt;
+	GSList *item;
+	GSList *val;
+	jobject schema;
+	err = NULL;
+	client_g = (GConfClient*)getPointerFromHandle(env, client);
+	str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
+	vt = (GConfValueType)type;
+	val = gconf_client_get_list(client_g, str, vt, &err);
+	(*env)->ReleaseStringUTFChars(env, key, str);
+	if (err != NULL)	{
+		updateStructHandle(env, exception, err, (JGFreeFunc) g_error_free);
+	}
 	int i = 0;
 	jclass aCls = NULL;
-	guint len;
+
 	jobjectArray ar;
 	if (NULL == val)
 		return NULL;
-	len = g_slist_length(val);
+		
 	if (vt == GCONF_VALUE_STRING)
 		aCls = (*env)->FindClass(env, "java/lang/String");
 	else if (vt == GCONF_VALUE_INT)
@@ -478,46 +503,49 @@ JNIEXPORT jobjectArray JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_
 		aCls = (*env)->FindClass(env, "org/gnu/gconf/ConfSchema");
 	if (NULL == aCls)
 		return NULL;
-	ar = (*env)->NewObjectArray(env, len, aCls, NULL);
-	for (i = 0; i < len; i++) {
-		GSList *item = g_slist_nth(val, i);
+	ar = (*env)->NewObjectArray(env, g_slist_length(val), aCls, NULL);
+	for (item = val; item != NULL; item = item->next, ++i) {
 		if (vt == GCONF_VALUE_STRING) {
 			jstring str = (*env)->NewStringUTF(env, item->data);
 			(*env)->SetObjectArrayElement(env, ar, i, str);
-		} else if (vt == GCONF_VALUE_INT) {
+		}
+		else if (vt == GCONF_VALUE_INT) {
 			jmethodID mid = (*env)->GetMethodID(env, aCls, "<init>", "(I)V");
 			jobject obj;
 			if (mid == NULL)
 				return NULL;
-			obj = (*env)->NewObject(env, aCls, mid, (jint)item->data);
+			obj = (*env)->NewObject(env, aCls, mid, (jint)GPOINTER_TO_INT(item->data));
 			(*env)->SetObjectArrayElement(env, ar, i, obj);
-		} else if (vt == GCONF_VALUE_FLOAT) {
+		}
+		else if (vt == GCONF_VALUE_FLOAT) {
 			jmethodID mid = (*env)->GetMethodID(env, aCls, "<init>", "(D)V");
 			jobject obj;
 			if (mid == NULL)
 				return NULL;
-			obj = (*env)->NewObject(env, aCls, mid, (jdouble*)item->data);
+			obj = (*env)->NewObject(env, aCls, mid, (jdouble)(*(gdouble*)(item->data)));
 			(*env)->SetObjectArrayElement(env, ar, i, obj);
-		} else if (vt == GCONF_VALUE_BOOL) {
+		}
+		else if (vt == GCONF_VALUE_BOOL) {
 			jmethodID mid = (*env)->GetMethodID(env, aCls, "<init>", "(Z)V");
 			jobject obj;
 			if (mid == NULL)
 				return NULL;
-			obj = (*env)->NewObject(env, aCls, mid, (jboolean)((gboolean)item->data));
+			obj = (*env)->NewObject(env, aCls, mid, (jboolean)(GPOINTER_TO_INT(item->data) != 0));
 			(*env)->SetObjectArrayElement(env, ar, i, obj);
-		} else if (vt == GCONF_VALUE_SCHEMA) {
-			jmethodID mid = (*env)->GetMethodID(env, aCls, "<init>", "(I)V");
+		}
+		else if (vt == GCONF_VALUE_SCHEMA) {
+		  //			jmethodID mid = (*env)->GetStaticMethodID(env, aCls, "getConfSchema", "(I)V");
+			jmethodID mid = (*env)->GetStaticMethodID(env, aCls, "getConfSchema", "(org/gnu/glib/Handle)org/gnu/glib/Handle");
 			jobject obj;
 			if (mid == NULL)
 				return NULL;
-			obj = (*env)->NewObject(env, aCls, mid, (jint)item->data);
+			
+			schema = getStructHandle(env, item->data,
+					(JGCopyFunc) gconf_schema_copy, (JGFreeFunc) gconf_schema_free);
+			obj = (*env)->CallStaticObjectMethod(env, aCls, mid, schema);
 			(*env)->SetObjectArrayElement(env, ar, i, obj);
 		}
 	}
-	(*env)->ReleaseStringUTFChars(env, key, str);
-	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
-	if (err != NULL)
-		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
 	return ar;
 }
 
@@ -526,21 +554,22 @@ JNIEXPORT jobjectArray JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_
  * Method:    gconf_client_get_pair
  */
 JNIEXPORT jboolean JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1pair
-  (JNIEnv *env, jclass cls, jobject client, jstring key, jint carType, jint cdrType, jintArray car, jintArray cdr, jintArray exception)
+  (JNIEnv *env, jclass cls, jobject client, jstring key, jint carType, jint
+  		cdrType, jintArray car, jintArray cdr, jobject exception)
 {
 	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
-	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
+	GError* err = NULL;
 	gpointer carloc = (gpointer)(*env)->GetIntArrayElements(env, car, NULL);
 	gpointer cdrloc = (gpointer)(*env)->GetIntArrayElements(env, cdr, NULL);
 	gboolean val = gconf_client_get_pair(client_g, str, (GConfValueType)carType, 
 			(GConfValueType)cdrType, carloc, cdrloc, &err);
 	(*env)->ReleaseStringUTFChars(env, key, str);
-	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
 	(*env)->ReleaseIntArrayElements(env, car, (jint*)carloc, 0);
 	(*env)->ReleaseIntArrayElements(env, cdr, (jint*)cdrloc, 0);
-	if (err != NULL)
-		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
+	if (err != NULL)	{
+		updateStructHandle(env, exception, err, (JGFreeFunc) g_error_free);
+	}
 	return (jboolean)val;
 }
 
@@ -549,16 +578,16 @@ JNIEXPORT jboolean JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1get_1pai
  * Method:    gconf_client_set_float
  */
 JNIEXPORT jboolean JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1set_1float
-  (JNIEnv *env, jclass cls, jobject client, jstring key, jdouble val, jintArray exception)
+  (JNIEnv *env, jclass cls, jobject client, jstring key, jdouble val, jobject exception)
 {
 	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
-	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
+	GError* err = NULL;
 	gboolean ret = gconf_client_set_float(client_g, str, (gdouble)val, &err);
 	(*env)->ReleaseStringUTFChars(env, key, str);
-	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
-	if (err != NULL)
-		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
+	if (err != NULL)	{
+		updateStructHandle(env, exception, err, (JGFreeFunc) g_error_free);
+	}
 	return (jboolean)ret;
 }
 
@@ -567,16 +596,16 @@ JNIEXPORT jboolean JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1set_1flo
  * Method:    gconf_client_set_int
  */
 JNIEXPORT jboolean JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1set_1int
-  (JNIEnv *env, jclass cls, jobject client, jstring key, jint val, jintArray exception)
+  (JNIEnv *env, jclass cls, jobject client, jstring key, jint val, jobject exception)
 {
 	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
-	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
+	GError* err = NULL;
 	gboolean ret = gconf_client_set_int(client_g, str, (gint)val, &err);
 	(*env)->ReleaseStringUTFChars(env, key, str);
-	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
-	if (err != NULL)
-		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
+	if (err != NULL)	{
+		updateStructHandle(env, exception, err, (JGFreeFunc) g_error_free);
+	}
 	return (jboolean)ret;
 }
 
@@ -585,18 +614,18 @@ JNIEXPORT jboolean JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1set_1int
  * Method:    gconf_client_set_string
  */
 JNIEXPORT jboolean JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1set_1string
-  (JNIEnv *env, jclass cls, jobject client, jstring key, jstring val, jintArray exception)
+  (JNIEnv *env, jclass cls, jobject client, jstring key, jstring val, jobject exception)
 {
 	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str1 = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
 	gchar *str2 = (gchar*)(*env)->GetStringUTFChars(env, val, NULL);
-	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
+	GError* err = NULL;
 	gboolean ret = gconf_client_set_string(client_g, str1, str2, &err);
 	(*env)->ReleaseStringUTFChars(env, key, str1);
 	(*env)->ReleaseStringUTFChars(env, key, str2);
-	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
-	if (err != NULL)
-		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
+	if (err != NULL)	{
+		updateStructHandle(env, exception, err, (JGFreeFunc) g_error_free);
+	}
 	return (jboolean)ret;
 }
 
@@ -605,16 +634,16 @@ JNIEXPORT jboolean JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1set_1str
  * Method:    gconf_client_set_bool
  */
 JNIEXPORT jboolean JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1set_1bool
-  (JNIEnv *env, jclass cls, jobject client, jstring key, jboolean val, jintArray exception)
+  (JNIEnv *env, jclass cls, jobject client, jstring key, jboolean val, jobject exception)
 {
 	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
-	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
+	GError* err = NULL;
 	gboolean ret = gconf_client_set_bool(client_g, str, (gboolean)val, &err);
 	(*env)->ReleaseStringUTFChars(env, key, str);
-	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
-	if (err != NULL)
-		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
+	if (err != NULL)	{
+		updateStructHandle(env, exception, err, (JGFreeFunc) g_error_free);
+	}
 	return (jboolean)ret;
 }
 
@@ -623,17 +652,17 @@ JNIEXPORT jboolean JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1set_1boo
  * Method:    gconf_client_set_schema
  */
 JNIEXPORT jboolean JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1set_1schema
-  (JNIEnv *env, jclass cls, jobject client, jstring key, jobject val, jintArray exception)
+  (JNIEnv *env, jclass cls, jobject client, jstring key, jobject val, jobject exception)
 {
 	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
 	GConfSchema* val_g = (GConfSchema*)getPointerFromHandle(env, val);
-	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
+	GError* err = NULL;
 	gboolean ret = gconf_client_set_schema(client_g, str, val_g, &err);
 	(*env)->ReleaseStringUTFChars(env, key, str);
-	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
-	if (err != NULL)
-		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
+	if (err != NULL)	{
+		updateStructHandle(env, exception, err, (JGFreeFunc) g_error_free);
+	}
 	return (jboolean)ret;
 }
 
@@ -642,11 +671,11 @@ JNIEXPORT jboolean JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1set_1sch
  * Method:    gconf_client_set_list
  */
 JNIEXPORT jboolean JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1set_1list
-  (JNIEnv *env, jclass cls, jobject client, jstring key, jint type, jobjectArray list, jintArray exception)
+  (JNIEnv *env, jclass cls, jobject client, jstring key, jint type, jobjectArray list, jobject exception)
 {
 	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
-	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
+	GError* err = NULL;
 	GConfValueType vt = (GConfValueType)type;
 	int i = 0;
 	jclass aCls = NULL;
@@ -654,12 +683,11 @@ JNIEXPORT jboolean JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1set_1lis
 	jfieldID fid;
 	jmethodID mid;
 	jsize len;
-	GSList* gslist;
+	GSList* gslist = NULL;
 	gpointer data = NULL;
 	gboolean ret;
 	
 	len = (*env)->GetArrayLength(env, list);
-	gslist = g_slist_alloc();
 	for (i = 0; i < len; i++) {
 		obj = (*env)->GetObjectArrayElement(env, list, i);
 		if (vt == GCONF_VALUE_STRING) {
@@ -695,21 +723,21 @@ JNIEXPORT jboolean JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1set_1lis
 			data = (gpointer)&value;
 		}
 		else if (vt == GCONF_VALUE_SCHEMA) {
-			jint value;
+			jobject value;
 			aCls = (*env)->FindClass(env, "org/gnu/gconf/ConfSchema");
-			fid = (*env)->GetFieldID(env, aCls, "handle", "I");
+			fid = (*env)->GetFieldID(env, aCls, "handle", "org/gnu/glib/Handle");
 			if (fid == NULL)
 				return (jboolean)JNI_FALSE;
-			value = (jint)(*env)->GetObjectField(env, obj, fid);
-			data = (gpointer)value;
+			value = (*env)->GetObjectField(env, obj, fid);
+			data = (gpointer)getPointerFromHandle(env, value);
 		}
 		gslist = g_slist_append(gslist, data);
 	}
-	ret = gconf_client_set_list(client_g, str, vt, gslist, &err);
+	ret = gconf_client_set_list(client_g, str, vt, gslist, &err);;
 	(*env)->ReleaseStringUTFChars(env, key, str);
-	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
-	if (err != NULL)
-		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
+	if (err != NULL)	{
+		updateStructHandle(env, exception, err, (JGFreeFunc) g_error_free);
+	}
 	return (jboolean)ret;
 }
 
@@ -718,17 +746,21 @@ JNIEXPORT jboolean JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1set_1lis
  * Method:    gconf_client_set_pair
  */
 JNIEXPORT jboolean JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1set_1pair
-  (JNIEnv *env, jclass cls, jobject client, jstring key, jint carType, jint cdrType, jint car, jint cdr, jintArray exception)
+  (JNIEnv *env, jclass cls, jobject client, jstring key, jint carType, jint
+  		cdrType, jobject car, jobject cdr, jobject exception)
 {
 	GConfClient* client_g = (GConfClient*)getPointerFromHandle(env, client);
 	gchar *str = (gchar*)(*env)->GetStringUTFChars(env, key, NULL);
-	GError* err = (GError*)(*env)->GetIntArrayElements(env, exception, NULL);
+	GError* err = NULL;
 	gboolean ret = gconf_client_set_pair(client_g, str, (GConfValueType)carType, 
-			(GConfValueType)cdrType, (gconstpointer)car, (gconstpointer)cdr, &err);
+					     (GConfValueType)cdrType,
+					     (gconstpointer)getPointerFromHandle(env,car), 
+					     (gconstpointer)getPointerFromHandle(env,cdr), 
+					     &err);
 	(*env)->ReleaseStringUTFChars(env, key, str);
-	(*env)->ReleaseIntArrayElements(env, exception, (jint*)err, 0);
-	if (err != NULL)
-		(*env)->SetIntArrayRegion(env, exception, 0, 1, (jint*)err);
+	if (err != NULL)	{
+		updateStructHandle(env, exception, err, (JGFreeFunc) g_error_free);
+	}
 	return (jboolean)ret;
 }
 
@@ -749,4 +781,3 @@ JNIEXPORT void JNICALL Java_org_gnu_gconf_ConfClient_gconf_1client_1value_1chang
 #ifdef __cplusplus
 }
 #endif
-#endif
