@@ -10,9 +10,10 @@
 #include <jni.h>
 #include <gconf/gconf.h>
 #include <jg_jnu.h>
+#include <gtk_java.h>
+
+#include "org_gnu_gconf_ConfValue.h"
 
-#ifndef _Included_org_gnu_gconf_ConfValue
-#define _Included_org_gnu_gconf_ConfValue
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -24,7 +25,8 @@ extern "C" {
 JNIEXPORT jobject JNICALL Java_org_gnu_gconf_ConfValue_gconf_1value_1new
   (JNIEnv *env, jclass cls, jint type)
 {
-	return getHandleFromPointer(env, gconf_value_new((GConfValueType) type));
+	return getStructHandle(env, gconf_value_new((GConfValueType) type), NULL,
+			(JGFreeFunc) gconf_value_free);
 }
 
 /*
@@ -35,7 +37,8 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gconf_ConfValue_gconf_1value_1copy
   (JNIEnv *env, jclass cls, jobject value)
 {
 	GConfValue* value_g = (GConfValue*)getPointerFromHandle(env, value);
-	return getHandleFromPointer(env, gconf_value_copy(value_g));
+	return getStructHandle(env, gconf_value_copy(value_g), NULL,
+			(JGFreeFunc) gconf_value_free);
 }
 
 /*
@@ -101,14 +104,20 @@ JNIEXPORT jint JNICALL Java_org_gnu_gconf_ConfValue_gconf_1value_1get_1list_1typ
 JNIEXPORT jobjectArray JNICALL Java_org_gnu_gconf_ConfValue_gconf_1value_1get_1list
   (JNIEnv *env, jclass cls, jobject value)
 {
-	GConfValue* value_g = (GConfValue*)getPointerFromHandle(env, value);
-	GSList* list = gconf_value_get_list(value_g);
-	int i = 0;
-	jclass aCls = NULL;
-	guint len;
+	GSList* it;
+	GConfValue* value_g;
+	GSList* list;
+	int i;
+	jclass aCls;
 	jobjectArray ar;
-	len = g_slist_length(list);
-	GConfValueType vt = gconf_value_get_list_type(value_g);
+	jobject schema;
+	GConfValueType vt;
+	value_g = (GConfValue*)getPointerFromHandle(env, value);
+	list = gconf_value_get_list(value_g);
+	i = 0;
+	aCls = NULL;
+	schema = NULL;
+	vt = gconf_value_get_list_type(value_g);
 	if (vt == GCONF_VALUE_STRING)
 		aCls = (*env)->FindClass(env, "java/lang/String");
 	else if (vt == GCONF_VALUE_INT)
@@ -121,39 +130,41 @@ JNIEXPORT jobjectArray JNICALL Java_org_gnu_gconf_ConfValue_gconf_1value_1get_1l
 		aCls = (*env)->FindClass(env, "org/gnu/gconf/ConfSchema");
 	if (NULL == aCls)
 		return NULL;
-	ar = (*env)->NewObjectArray(env, len, aCls, NULL);
-	for (i = 0; i < len; i++) {
-		GSList *item = g_slist_nth(list, i);
+	ar = (*env)->NewObjectArray(env, g_slist_length(list), aCls, NULL);
+	for (it = list; it != NULL; it = it->next, ++i)	{
 		if (vt == GCONF_VALUE_STRING) {
-			jstring str = (*env)->NewStringUTF(env, item->data);
+			jstring str = (*env)->NewStringUTF(env, it->data);
 			(*env)->SetObjectArrayElement(env, ar, i, str);
 		} else if (vt == GCONF_VALUE_INT) {
 			jmethodID mid = (*env)->GetMethodID(env, aCls, "<init>", "(I)V");
 			jobject obj;
 			if (mid == NULL)
 				return NULL;
-			obj = (*env)->NewObject(env, aCls, mid, (jint)item->data);
+			obj = (*env)->NewObject(env, aCls, mid, (jint)GPOINTER_TO_INT(it->data));
 			(*env)->SetObjectArrayElement(env, ar, i, obj);
 		} else if (vt == GCONF_VALUE_FLOAT) {
 			jmethodID mid = (*env)->GetMethodID(env, aCls, "<init>", "(D)V");
 			jobject obj;
 			if (mid == NULL)
 				return NULL;
-			obj = (*env)->NewObject(env, aCls, mid, (jdouble*)item->data);
+			obj = (*env)->NewObject(env, aCls, mid, (jdouble*)it->data);
 			(*env)->SetObjectArrayElement(env, ar, i, obj);
 		} else if (vt == GCONF_VALUE_BOOL) {
 			jmethodID mid = (*env)->GetMethodID(env, aCls, "<init>", "(Z)V");
 			jobject obj;
 			if (mid == NULL)
 				return NULL;
-			obj = (*env)->NewObject(env, aCls, mid, (jboolean)((gboolean)item->data));
+			obj = (*env)->NewObject(env, aCls, mid, (jboolean)(GPOINTER_TO_INT(it->data) != 0));
 			(*env)->SetObjectArrayElement(env, ar, i, obj);
 		} else if (vt == GCONF_VALUE_SCHEMA) {
-			jmethodID mid = (*env)->GetMethodID(env, aCls, "<init>", "(I)V");
+			jmethodID mid = (*env)->GetStaticMethodID(env, aCls, "getConfSchema", "(I)V");
 			jobject obj;
 			if (mid == NULL)
 				return NULL;
-			obj = (*env)->NewObject(env, aCls, mid, (jint)item->data);
+			
+			schema = getStructHandle(env, it->data,
+					(JGCopyFunc) gconf_schema_copy, (JGFreeFunc) gconf_schema_free);
+			obj = (*env)->CallStaticObjectMethod(env, aCls, mid, schema);
 			(*env)->SetObjectArrayElement(env, ar, i, obj);
 		}
 	}
@@ -168,7 +179,8 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gconf_ConfValue_gconf_1value_1get_1car
   (JNIEnv *env, jclass cls, jobject value)
 {
 	GConfValue* value_g = (GConfValue*)getPointerFromHandle(env, value);
-	return getHandleFromPointer(env, gconf_value_get_car(value_g));
+	return getStructHandle(env, gconf_value_get_car(value_g),
+		(JGCopyFunc) gconf_value_copy, (JGFreeFunc) gconf_value_free);
 }
 
 /*
@@ -179,7 +191,8 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gconf_ConfValue_gconf_1value_1get_1cdr
   (JNIEnv *env, jclass cls, jobject value)
 {
 	GConfValue* value_g = (GConfValue*)getPointerFromHandle(env, value);
-	return getHandleFromPointer(env, gconf_value_get_cdr(value_g));
+	return getStructHandle(env, gconf_value_get_cdr(value_g),
+			(JGCopyFunc) gconf_value_copy, (JGFreeFunc) gconf_value_free);
 }
 
 /*
@@ -201,7 +214,8 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gconf_ConfValue_gconf_1value_1get_1schema
   (JNIEnv *env, jclass cls, jobject value)
 {
 	GConfValue* value_g = (GConfValue*)getPointerFromHandle(env, value);
-	return getHandleFromPointer(env, gconf_value_get_schema(value_g));
+	return getStructHandle(env, gconf_value_get_schema(value_g),
+			(JGCopyFunc) gconf_schema_copy, (JGFreeFunc) gconf_schema_free);
 }
 
 /*
@@ -380,4 +394,3 @@ JNIEXPORT jstring JNICALL Java_org_gnu_gconf_ConfValue_gconf_1value_1to_1string
 #ifdef __cplusplus
 }
 #endif
-#endif
